/*
    (C) 1995-96 AROS - The Amiga Research OS
    $Id: loadseg.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc: DOS function LoadSeg()
    Lang: english

    Revision 1.14
      LoadSeg now simply calls InternalLoadSeg() with the
      array of functions
*/
#include <proto/exec.h>
#include <exec/memory.h>
#include <dos/dos.h>
#include <dos/dosextens.h>
#include <proto/dos.h>
#include "dos_intern.h"

/* iaint: Sigh, I'm getting sick of this...
*/
#undef DEBUG
#define DEBUG 0
#	include <aros/debug.h>

#ifdef __MORPHOS__
extern const struct EmulLibEntry Dos_Read;
extern const struct EmulLibEntry Exec_AllocMem;
extern const struct EmulLibEntry Exec_FreeMem;
#else
extern LONG Dos_Read(BPTR, APTR, LONG, struct DosLibrary *);
extern void * Exec_AllocMem(ULONG, ULONG, struct ExecBase *);
extern void Exec_FreeMem(APTR, ULONG, struct ExecBase *);
#endif

/*****************************************************************************

    NAME */
#include <proto/dos.h>

	AROS_LH3(BPTR, LoadSeg,

/*  SYNOPSIS */
        AROS_LHA(STRPTR, name, D1),
	AROS_LHA(BPTR, table, D2),
	AROS_LHA(BPTR, fh, D3),

/*  LOCATION */
        struct DosLibrary *, DOSBase, 25, Dos)

/*  FUNCTION
        Loads an executable file into memory. Each hunk of the loadfile
        is loaded into his own memory section and a handle on all of them
        is returned. The segments can be freed with UnLoadSeg().

    INPUTS
        name - NUL terminated name of the file.

    RESULT
        Handle to the loaded executable or 0 if the load failed.
        IoErr() gives additional information in that case.

    NOTES
        This function is built on top of InternalLoadSeg()

    EXAMPLE

    BUGS

    SEE ALSO
        UnLoadSeg()

    INTERNALS

    HISTORY
        29-10-95    digulla automatically created from
                            dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
  AROS_LIBFUNC_INIT
  AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

  static const void (* const FunctionArray[])(void) =
  {
    (void *)&Dos_Read,
    (void *)&Exec_AllocMem,
    (void *)&Exec_FreeMem
  };

  BPTR file, segs=0;

  DEBUG_LOADSEG(dprintf("LoadSeg: name = \"%s\", table = $%lx, file = $%lx\n", name ? name : (CONST_STRPTR) "<NULL>", table, fh));

  FAILFORTASKNP(return 0;)

  if (name)
  {
    /* Open the file */
    file = Open (name,MODE_OLDFILE);
    table = NULL;
  }
  else
    file = fh; /* Overlay */

  if (file)
  {
    segs = InternalLoadSeg (file, table, (void *)FunctionArray, NULL);

    if (segs)
      SetIoErr (0);
#if DEBUG > 1
    else
      bug ("Loading <%s> failed\n", name);
#endif
    if (segs < 0) /* Overlay */
      segs = -segs; 
    else
      Close (file);
  }

  DEBUG_LOADSEG(dprintf("LoadSeg: res = $%lx\n", segs));

#ifdef __MORPHOS__
  REG_D1 = segs;
#endif

  /* And return */
  return segs;

  AROS_LIBFUNC_EXIT
} /* LoadSeg */
